#include <Adafruit_GFX.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_ILI9341.h>
#include <Adafruit_STMPE610.h>

// This is calibration data for the raw touch data to the screen coordinates
#define TS_MINX 150
#define TS_MINY 130
#define TS_MAXX 3800
#define TS_MAXY 4000

#define STMPE_CS 8
Adafruit_STMPE610 ts = Adafruit_STMPE610(STMPE_CS);
#define TFT_CS 10
#define TFT_DC 9
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC, -1, false);

#define BOXSIZE 50
int oldcolor, currentcolor;
boolean RecordOn = false;
int x = 0;

uint8_t uCurrentScreen = 0;
enum {
  SCREEN_MAIN=0, SCREEN_SECOND};

//---------------------------------------------------------
// Some hacks for switching SPI registers
enum {
  SPIR_TFT=0, SPIR_TS, SPIR_CNT};
extern void SaveSPIRegs(uint8_t spir);
extern void RestoreSPIRegs(uint8_t spir);
//-----------------------------------------------------------

void setup(void)
{
  delay(2000);
  Serial.begin(9600);
  PrintPinsConfig();
  tft.begin();
  PrintPinsConfig();
  SaveSPIRegs(SPIR_TFT);
  if (!ts.begin()) { 

    Serial.println("Unable to start touchscreen.");
    delay(2000);
  } 
  else { 

    Serial.println("Touchscreen started."); 
    delay(2000);
  }
  PrintPinsConfig();
  SaveSPIRegs(SPIR_TS);

  RestoreSPIRegs(SPIR_TFT);
  PrintPinsConfig();

  SetDisplayMainScreen();


}

void loop()
{
  // See if there's any  touch data for us
  //if (ts.bufferEmpty()) {
  // return;
  //}

  // You can also wait for a touch
  RestoreSPIRegs(SPIR_TS);
  if (! ts.touched()) {
    return;
  }
  switch (uCurrentScreen) {
  case SCREEN_MAIN: 
    ProcessTouchMainScreen();
    break;
  case SCREEN_SECOND: 
    ProcessTouchSecondScreen();
    break;
  }
}  

//---------------------------------------------------------
// Some hacks for switching SPI registers
uint8_t   g_icurSPIR = 0xff;  // Set to something that won't be
uint32_t  g_aluSPI0_CTAR0[SPIR_CNT];
uint32_t  g_alsuSPI0_PIN10_CONFIG[SPIR_CNT];  // pin 10 CS gets mucked with...
uint32_t  g_aluSPI0_MCR[SPIR_CNT];

void SaveSPIRegs(uint8_t spir) {
  g_aluSPI0_MCR[spir] = SPI0_MCR;
  g_aluSPI0_CTAR0[spir] = SPI0_CTAR0;
  g_alsuSPI0_PIN10_CONFIG[spir] = CORE_PIN10_CONFIG;
}

void RestoreSPIRegs(uint8_t spir) {
  if (g_icurSPIR != spir) {  
    // Need to wait until all outputs are done... 
    // Wait until hardware queue is empty...
    uint16_t wTimeout = 0xffff;
    while (((SPI0.SR) & (15 << 12)) && (--wTimeout)) ; // wait until empty

    // May want to wait until actual output is done? also may need to turn off SPI before change?
    // But for now simply try restoring the register(s)
    SPI0.MCR = SPI_MCR_MDIS | SPI_MCR_HALT;
    SPI0_CTAR0 = g_aluSPI0_CTAR0[spir];
    SPI0_MCR = g_aluSPI0_MCR[spir];
    CORE_PIN10_CONFIG = g_alsuSPI0_PIN10_CONFIG[spir];
    g_icurSPIR = spir;
  }
}
//-----------------------------------------------------------

void PrintPinsConfig(void) {
  Serial.print(CORE_PIN8_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN9_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN10_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN11_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN12_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN13_CONFIG, HEX);
  Serial.println(" ");
  //  Serial.print(SPI0.MCR, HEX);
  //  Serial.print(" ");
  //  uint32_t ulSR = SPI0.SR;
  //  Serial.println(SPI0.SR, HEX);
}

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------

void SetDisplayMainScreen(){

  tft.fillScreen(ILI9341_WHITE);
  Serial.println("WHITE");
//  delay(2000);

  tft.fillRoundRect(12, 14, BOXSIZE, BOXSIZE,9, ILI9341_RED);
  tft.fillRect(12, 25+BOXSIZE, BOXSIZE, BOXSIZE, ILI9341_YELLOW);
  tft.fillRect(12, 35+BOXSIZE*2, BOXSIZE, BOXSIZE, ILI9341_GREEN);
  tft.fillRect(12, 45+BOXSIZE*3, BOXSIZE, BOXSIZE, ILI9341_CYAN);
  tft.fillRoundRect(12, 55+BOXSIZE*4, BOXSIZE, BOXSIZE, 7,ILI9341_BLUE);
  tft.fillRect(74, 35+BOXSIZE*2, BOXSIZE, BOXSIZE, ILI9341_MAGENTA);

  tft.fillRoundRect(180, 14, BOXSIZE, BOXSIZE,9, ILI9341_BLACK);
  tft.fillTriangle(215, 45, 195, 45, 204, 28, ILI9341_WHITE);

  // Set which screen is current
  uCurrentScreen = SCREEN_MAIN;

}
//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------

void SetDisplaySecondScreen(){  

  tft.fillScreen(ILI9341_BLUE);

  tft.fillRoundRect(180, 55+BOXSIZE*4, BOXSIZE, BOXSIZE,9, ILI9341_BLACK);
  tft.fillTriangle(215, 270, 195, 270, 204, 290, ILI9341_WHITE);
  tft.fillRoundRect(180, 14, BOXSIZE, BOXSIZE,9, ILI9341_BLACK);
  tft.fillTriangle(215, 45, 195, 45, 204, 28, ILI9341_WHITE);

  uCurrentScreen = SCREEN_SECOND;
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchSecondScreen(){  

  TS_Point p = ts.getPoint();

  // Serial.print("X = "); Serial.print(p.x);
  //Serial.print("\tY = "); Serial.print(p.y);
  //Serial.print("\tPressure = "); Serial.println(p.z);  


  // Scale from ~0->4000 to tft.width using the calibration #'s
  p.x = map(p.x, TS_MINX, TS_MAXX, 0, tft.width());
  p.y = map(p.y, TS_MINY, TS_MAXY, 0, tft.height());

  if (p.x >= 195 && p.x <= 215) {
    RestoreSPIRegs(SPIR_TFT);
    Serial.print("("); 
    Serial.print(p.x);
    Serial.print(", "); 
    Serial.print(p.y);
    Serial.println(")");
    if (p.y >= 242 && p.y <= 292) {
      SetDisplayMainScreen();
    }
    if (p.y >= 14 && p.y <= 60){
      currentcolor = ILI9341_BLACK;
      tft.fillTriangle(215, 45, 195, 45, 204, 28, ILI9341_CYAN);
    }
  }
}



//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchMainScreen(){

  // Retrieve a point  
  TS_Point p = ts.getPoint();


  // Serial.print("X = "); Serial.print(p.x);
  //Serial.print("\tY = "); Serial.print(p.y);
  //Serial.print("\tPressure = "); Serial.println(p.z);  


  // Scale from ~0->4000 to tft.width using the calibration #'s
  p.x = map(p.x, TS_MINX, TS_MAXX, 0, tft.width());
  p.y = map(p.y, TS_MINY, TS_MAXY, 0, tft.height());


  // Serial.print("("); Serial.print(p.x);
  // Serial.print(", "); Serial.print(p.y);
  //Serial.println(")");


  if (p.x >= 20 && p.x <= 70) {
    RestoreSPIRegs(SPIR_TFT);
    oldcolor = currentcolor;

    if (p.y >= 15 && p.y <= 62) { 
      currentcolor = ILI9341_RED; 
      tft.fillRoundRect(12, 14, BOXSIZE, BOXSIZE,9, ILI9341_BLACK);
      tft.fillRoundRect(17, 19, 40, 40,9, ILI9341_RED);
      //tft.drawRoundRect(12, 14, BOXSIZE, BOXSIZE,9, ILI9341_BLACK);
    } 
    else if (p.y >= 77 && p.y <= 116) {
      currentcolor = ILI9341_YELLOW;
      tft.drawRect(12, 25+BOXSIZE, BOXSIZE, BOXSIZE, ILI9341_BLACK);
    } 
    else if (p.y >= 125 && p.y <= 170) {
      currentcolor = ILI9341_GREEN;
      tft.drawRect(12, 35+BOXSIZE*2, BOXSIZE, BOXSIZE, ILI9341_BLACK);
    } 
    else if (p.y >= 182 && p.y <= 230) {
      currentcolor = ILI9341_CYAN;
      tft.drawRect(12, 45+BOXSIZE*3, BOXSIZE, BOXSIZE, ILI9341_BLACK);
    } 
    else if (p.y >= 242 && p.y <= 292) {
      currentcolor = ILI9341_BLUE;
      tft.fillRoundRect(12, 55+BOXSIZE*4, BOXSIZE, BOXSIZE,7,ILI9341_BLACK);
      tft.fillRoundRect(16, 60+BOXSIZE*4, BOXSIZE-10, BOXSIZE-11, 7,ILI9341_BLUE);

    } 
  }
  else if (p.x >= 82 && p.x <= 132) {
    RestoreSPIRegs(SPIR_TFT);
    oldcolor = currentcolor;
    if (p.y >= 125 && p.y <= 170) {
      currentcolor = ILI9341_MAGENTA;
      tft.drawRect(74, 35+BOXSIZE*2, BOXSIZE, BOXSIZE, ILI9341_BLACK);
    }
  }
  else if (p.x >= 195 && p.x <= 215) {
    RestoreSPIRegs(SPIR_TFT);
    oldcolor = currentcolor;
    if (p.y >= 14 && p.y <= 60) {
      currentcolor = ILI9341_BLACK;
      SetDisplaySecondScreen();
      return;
    }
  }

  if (oldcolor != currentcolor) {
    RestoreSPIRegs(SPIR_TFT);
    if (oldcolor == ILI9341_RED) 
      tft.fillRoundRect(12, 14, BOXSIZE, BOXSIZE,9, ILI9341_RED);
    if (oldcolor == ILI9341_YELLOW) 
      tft.fillRect(12, 25+BOXSIZE, BOXSIZE, BOXSIZE, ILI9341_YELLOW);
    if (oldcolor == ILI9341_GREEN) 
      tft.fillRect(12, 35+BOXSIZE*2, BOXSIZE, BOXSIZE, ILI9341_GREEN);
    if (oldcolor == ILI9341_CYAN) 
      tft.fillRect(12, 45+BOXSIZE*3, BOXSIZE, BOXSIZE, ILI9341_CYAN);
    if (oldcolor == ILI9341_BLUE) 
      tft.fillRoundRect(12, 55+BOXSIZE*4, BOXSIZE, BOXSIZE, 7,ILI9341_BLUE);
    if (oldcolor == ILI9341_MAGENTA) 
      tft.fillRect(74, 35+BOXSIZE*2, BOXSIZE, BOXSIZE, ILI9341_MAGENTA);
    if (oldcolor == ILI9341_BLACK) 
      tft.fillTriangle(215, 45, 195, 45, 204, 28, ILI9341_WHITE);
  }
}




